#ifndef FLY_DE_H
#define FLY_DE_H

#include <fly/flytaggedbool.h>

#include <utime.h>

#include <vector>
#include <string>

using Display = struct _XDisplay;
using Window = unsigned long;

#ifdef FLY_ORG_NAME
#define FLY_ORG FLY_ORG_NAME
#else
#define FLY_ORG "rusbitech"
#endif

#define FLYWM_LOCKER_WINDOW_NAME "FLYWM_LOCKER_WINDOW"

//compatible with kdeworkspace.h
enum FlyShutdownConfirm {
    ShutdownConfirmDefault =-1,// Obey the user's confirmation setting.
    ShutdownConfirmNo      = 0,// Don't confirm, shutdown without asking.
    ShutdownConfirmYes     = 1 // Always confirm, ask even if the user turned it off.
 };
enum FlyShutdownType {
	ShutdownTypeUnknown =-4,
	ShutdownTypeCancel  =-3,
	ShutdownTypeLock    =-2,
	ShutdownTypeDefault =-1,// Select previous action or the default if it's the first time.
    ShutdownTypeNone    = 0,// Only log out
    ShutdownTypeReboot  = 1,// Log out and reboot the machine.
    ShutdownTypeHalt    = 2,// Log out and halt the machine.
    ShutdownTypeLogout  = 3 // Temporary brain damage. Don't use. Same as ShutdownTypeNone

};
enum FlyShutdownMode {
    ShutdownModeDefault    = -1,// Select previous mode or the default if it's the first time.
    ShutdownModeSchedule   = 0, // Schedule a shutdown (halt or reboot) for the time all active have exited.
    ShutdownModeTryNow     = 1,	// Shut down, if no sessions are active. Otherwise do nothing.
    ShutdownModeForceNow   = 2, // Force shutdown. Kill any possibly active sessions
    ShutdownModeInteractive= 3  //Pop up a dialog asking the user what to do if sessions are
};

Window FindToplevelWindowByName(Display *dpy,Window win,const char *winName);

const char *GetOrganization();

//get message locale (calls GetFlyWMMessage to find and load message file)
const char *GetFlyWMMessageLocale();

//get message by index
char *GetFlyWMMessage(int index=-1,const char *defaultMsg="");

std::vector<std::string> GetDMSessionsList();
int GetDMReserveNum();


// WM and DM command interface (don't forget to finish command line with '\n')
std::pair<std::string, bool> GetFlyWMFIFOPath(Display *);  // false if path is not accessible
std::pair<std::string, bool> FindFlyWMFIFOPath(Display *); // false if path is not accessible
std::pair<std::string, bool> GetFlyDMFIFOPath(Display *);  // false if path is not accessible
bool SendCommandToWM(Display *,const char *buf); //if buf=NULL or empty just check socket/fifo availability
bool SendCommandToDM(Display *,const char *buf); //if buf=NULL or empty just check socket/fifo availability

//this enum and function borrowed from Qt4
enum WhatDesktopEnvironment {
    DE_UNKNOWN,
    DE_KDE,
    DE_GNOME,
    DE_CDE,
    DE_4DWM, DE_FLY
};
//in Qt4 use QX11Info::display(), QX11Info::appRootWindow() as arguments
WhatDesktopEnvironment whatDE(Display *dpy, Window rootWindow);

enum WhatFlySession {
    FLY_UNKNOWN,
    FLY_FAILSAFE,
    FLY_DESKTOP,
    FLY_TABLET,
    FLY_MINI,
    FLY_MOBILE,
    FLY_WESTON,
    FLY_KIOSK,
    FLY_TABLET_KIOSK,
};

bool isFlyKioskSession();

WhatFlySession whatFlySession(); //check $DESKTOP_SESSION

//get session name, just returns getenv("DESKTOP_SESSION")
const char * flySessionName();

//returns config suffix for most fly-wm config files, never returns NULL
std::string flySessionConfigSuffix(WhatFlySession stype=FLY_UNKNOWN);

//save session name in ~/.fly/last-desktop-session-name
bool flySessionSaveName(const char *name = nullptr);
//get session name from ~/.fly/last-desktop-session-name, never returns NULL
std::string flySessionGetSavedName();

void RenameDesktopShortcutCoord(const char *pathNameOld, const char *newName, Display *dpy);

//save in desktop .directory the section: [IconPosition:fn] with X=x and Y=y fields
// to allow desktop item positioning by fly-wm
bool SaveDesktopItemPosition(const char *fn,const int x,const int y);
bool RemoveDesktopItemPosition(const char *fn);

using UpdateOnly = Fly::tagged_bool<class UpdateOnly_tag>;
void CreateStartMenu(const char *menuDir, UpdateOnly updateOnly = UpdateOnly{false});

void syncDesktopEntryTime(bool initial, const std::string& filePath,  const struct utimbuf *tbuf);
void syncUserDesktopEntriesTime(bool initial, const struct utimbuf *tbuf);

#endif
