#ifndef FLY_DE_API_H
#define FLY_DE_API_H

#include <fly/flytaggedbool.h>

#include <string>

using Display = struct _XDisplay;

enum FlyDir {
    //wm specific
    WM_DIR,
    //user specific
    USER_TMP_DIR,       USER_BASE_DIR,    USER_THEME_DIR,     USER_START_MENU_DIR, 
    USER_CPL_DIR, //obsolete!
    USER_DESKTOP_DIR, USER_DOCUMENTS_DIR, USER_TRASH_DIR,   
    FLY_WORK_DIR, //deprecated, use USER_TMP_DIR
    //app specific (shared)
    APP_SHARED_DIR,  APP_TRANS_DIR, APP_DOCS_DIR, APP_DOCS_HTML_DIR,
    WM_SOUNDS_DIR, WM_IMAGES_DIR, WM_KEYMAPS_DIR,
    //user specific more
    USER_TOOLBAR_DIR, USER_HOME_DIR,
    //added from xdg-user-dirs
    USER_VIDEOS_DIR, USER_MUSIC_DIR, USER_PICTURES_DIR, USER_DOWNLOAD_DIR, USER_PUBLICSHARE_DIR, USER_TEMPLATES_DIR,
    //others
    USER_AUTOSTART_MENU_DIR, //fredesktop.org's autostart dir for user
    USER_CACHE_DIR,
    WM_DECOR_DIR,
    WM_SHARED_DESKTOP_DIR, WM_SHARED_STARTMENU_DIR
};

//Get Fly specific and standard directory paths
// returns a ref to a static string like "~/.fly[/...]"
const std::string &flyGetPath(FlyDir d);

//send command to fly-wm for example "FLYWM_SHUTDOWN\n" and many others
bool flySendCommandToWM(Display *,const char *buf); //if buf=0 or empty just check socket/fifo availability

//check if 'appname.desktop' installed and enabled(valid) in user's/system autostart
bool flyIsAutostartEnabled(const char *appname);
//install and-or enable/disable 'appname.desktop' in user's/system autostart
using DeleteIfDisabled = Fly::tagged_bool<class DeleteIfDisabled_tag>;
bool flyEnableAutostart(const char *appname, bool enable, const char *newExec = nullptr,
                        DeleteIfDisabled deleteIfDisabled = DeleteIfDisabled{true});
//check if 'appname.desktop' installed and enabled and if so then return fill path
std::string flyAutostartEnabledPath(const char *appname);
#endif
