#ifndef FLY_DESKTOP_ENTRY
#define FLY_DESKTOP_ENTRY

#include <fly/flytaggedbool.h>

#include <vector>
#include <string>

#include <stdlib.h>

//#if defined(__linux__) || defined(_linux_) || defined(linux)
//#include <linux/limits.h> //for PATH_MAX, NAME_MAX
//#else
#include <limits.h>
//#endif

//#include "flyicontheme.h"
//#define ICON_CONTEXT_IN_DTOP_ENTRY 1

#define SOME_WORD_SIZE             64

//#define X_FLY_COPY	"X-FLY-COPY"

//class represents desktop entry file (see /usr[/local]/share/applications/*.desktop)
class FlyDesktopActionPrivate;
class FlyDesktopAction
{
public:
    FlyDesktopAction(std::string id);
    FlyDesktopAction(const FlyDesktopAction&);
    FlyDesktopAction(FlyDesktopAction&&) noexcept;
    ~FlyDesktopAction();

    FlyDesktopAction& operator=(const FlyDesktopAction&);
    FlyDesktopAction& operator=(FlyDesktopAction&&) noexcept;
    bool  operator==(const FlyDesktopAction&) const;

    const std::string &id() const;
    void setId(std::string id);

    const std::string &name() const;
    void setName(std::string name);
    const std::string &localName() const;
    void setLocalName(std::string localName);
    const std::string &localNameOrFallback() const;

    const std::string &comment() const;
    void setComment(std::string comment);
    const std::string &localComment() const;
    void setLocalComment(std::string localComment);
    const std::string &localCommentOrFallback() const;

    const std::string &icon() const;
    void setIcon(std::string icon);
    const std::string &localIcon() const;
    void setLocalIcon(std::string localIcon);
    const std::string &localIconOrFallback() const;

    const std::string &path() const;
    void setPath(std::string path);

    const std::string &exec() const;
    void  setExec(std::string exec);

    const std::vector<std::string> &onlyShowIn() const;
    void setOnlyShowIn(std::vector<std::string> osh);

    const std::vector<std::string> &notShowIn() const;
    void setNotShowIn(std::vector<std::string> nsh);

private:
    FlyDesktopActionPrivate * p;
};

class FlyDesktopEntryPrivate;
class FlyDesktopEntry 
{
public:
    FlyDesktopEntry(std::string filePath = std::string());
    ~FlyDesktopEntry();

    FlyDesktopEntry(const FlyDesktopEntry &other) = delete;
    FlyDesktopEntry &operator=(const FlyDesktopEntry &other) = delete;

    FlyDesktopEntry(FlyDesktopEntry &&other) noexcept;
    FlyDesktopEntry &operator=(FlyDesktopEntry &&other) noexcept;

    bool identicalTo(const FlyDesktopEntry&) const;
    void copyFrom(const FlyDesktopEntry &from);
    void inheritFrom(const FlyDesktopEntry &from);
    void overrideFrom(const FlyDesktopEntry &from);

    // set/get path to read()/save()
    const std::string &filePath() const;
    void setFilePath(std::string filePath);

    bool  load(); //reload *.desktop file
    using SaveEmptyFields = Fly::tagged_bool<class SaveEmptyFields_tag>;
    bool  save(/*int x=-1,int y=-1,*/ SaveEmptyFields saveEmpty = SaveEmptyFields{false}); //save, dx,dy - coords on desktop
    bool  saveAs(std::string filePath, SaveEmptyFields saveEmpty = SaveEmptyFields{false});
	bool  savePositionOnDesktop(int dx,int dy);

    bool  isValid() const;
    bool  isLoaded()const;

    const std::string &name() const;
    void setName(std::string name);
    const std::string &localName() const;
    void setLocalName(std::string localName);
    const std::string &localNameOrFallback() const;

    const std::string &genericName() const;
    void setGenericName(std::string genericName);
    const std::string &localGenericName() const;
    void setLocalGenericName(std::string localGenericName);
    const std::string &localGenericNameOrFallback() const;

    //"Application", "Link", "MimeType", "Directory", "Service", "ServiceType"
    const std::string &type() const;
    void setType(std::string type);

    const std::string &tryExec() const;
    void setTryExec(std::string tryExec);

    bool hasNoDisplay() const;
    bool noDisplay() const;
    void setNoDisplay(bool n);
   
    const std::string &comment() const;
    void setComment(std::string comment);
    const std::string &localComment() const;
    void setLocalComment(std::string localComment);
    const std::string &localCommentOrFallback() const;

    const std::string &exec() const;
    void setExec(std::string exec);

    const std::vector<FlyDesktopAction> &actions() const;
    void setActions(std::vector<FlyDesktopAction> actions);

    const std::string &icon() const;
    void setIcon(std::string icon);
    const std::string &localIcon() const;
    void setLocalIcon(std::string localIcon);
    const std::string &localIconOrFallback() const;

    bool hasHidden() const;
    bool hidden() const;
    void setHidden(bool h);

    const std::string &path() const;
    void setPath(std::string path);

    bool hasTerminal() const;
    bool terminal() const;
    void setTerminal(bool t);

    bool hasStartupNotify() const;
    bool startupNotify() const;
    void setStartupNotify(bool t);

    const std::string &startupWMClass() const;
    void setStartupWMClass(std::string wmClass);

    bool hasPrefersNonDefaultGPU() const;
    bool prefersNonDefaultGPU() const;
    void setPrefersNonDefaultGPU(bool t);

    bool hasSingleMainWindow() const;
    bool singleMainWindow() const;
    void setSingleMainWindow(bool t);

    const std::vector<std::string> &mimeType() const;
    void setMimeType(std::vector<std::string> mmt);

    const std::vector<std::string> &sortOrder() const;
    void setSortOrder(std::vector<std::string> so);

    const std::string &url() const;
    void setUrl(std::string url);

    const std::vector<std::string> &categories() const;
    void setCategories(std::vector<std::string> ca);

    const std::vector<std::string> &onlyShowIn() const;
    void setOnlyShowIn(std::vector<std::string> osh);

    const std::vector<std::string> &notShowIn() const;
    void setNotShowIn(std::vector<std::string> nsh);

    enum SuitableCode {
        Suitable        = 0,
        ExeEmpty        = 1,
        ExeMalformed    = 2,
        ExeNotExists    = 3,
        ExeNotFound     = 4,
        NoOrigDtopFile  = 5,
        TryExeNotExists = 7,
        TryExeNotFound  = 8,
        UrlEmpty        = 9,
        NotAdmin        = 10,
        NotMode         = 11,
    };
    enum DisplayableCode {
        NoDisplay   = -3,
        NotForFly   = -2,
        Hidden      = -1,
        Displayable = 0
    };

    using AdminOnly = Fly::tagged_bool<class AdminOnly_tag>;
    static bool isExecSuitable   (const char *execField, AdminOnly admin_only = AdminOnly{false}, SuitableCode *retCode=nullptr);
    static bool isTryExecSuitable(const char *execField, SuitableCode *retCode=nullptr);

    bool isSuitable(SuitableCode *retCode = nullptr) const;
    bool isDisplayable(DisplayableCode *retCode = nullptr) const;
    bool isForFly(DisplayableCode *retCode = nullptr) const;


    const std::string &version() const;
    void setVersion(std::string version);

    const std::vector<std::string> &keywords() const;
    void setKeywords(std::vector<std::string> kws);
    const std::vector<std::string> &localKeywords() const;
    void setLocalKeywords(std::vector<std::string> kws);
    const std::vector<std::string> &localKeywordsOrFallback() const;

	//get/set unknown key-values pairs
    const std::vector<std::pair<std::string, std::string>> &unknowns() const;
    void setUnknowns(std::vector<std::pair<std::string, std::string>> u);
    std::string getUnknownValue(const char *key) const;


private:
    FlyDesktopEntryPrivate * p;
};

// search *.desktop in  /usr[/local]/share/applications
// for MimeType field 'mime'
using AllDesktopEntries = Fly::tagged_bool<class AllDesktopEntries_tag>;
std::vector<std::string> GetDesktopEntryPathForMime(const char *mime, AllDesktopEntries all = AllDesktopEntries{false});
std::vector<FlyDesktopEntry> GetDesktopEntryForMime(const char *mime, AllDesktopEntries all = AllDesktopEntries{false});

//get full abs. path for desktop entry file name (appname.desktop)
std::string GetDesktopEntryFullPath(const char *entry);

//get all mime type aliases
std::vector<std::string> GetMimeAliases(const char *mimeName);


//$XDG_CONFIG_HOME/$desktop-mimeapps.list	            user overrides, desktop-specific (for advanced users)
//$XDG_CONFIG_HOME/mimeapps.list	                    user overrides (recommended location for user configuration GUIs)
//$XDG_CONFIG_DIRS/$desktop-mimeapps.list	            sysadmin and ISV overrides, desktop-specific
//$XDG_CONFIG_DIRS/mimeapps.list	                    sysadmin and ISV overrides
//$XDG_DATA_HOME/applications/$desktop-mimeapps.list	for completeness, deprecated, desktop-specific
//$XDG_DATA_HOME/applications/mimeapps.list	            for compatibility, deprecated
//$XDG_DATA_DIRS/applications/$desktop-mimeapps.list	distribution-provided defaults, desktop-specific
//$XDG_DATA_DIRS/applications/mimeapps.list             distribution-provided defaults
//In this table, $desktop is one of the names of the current desktop, lowercase (for instance, kde, gnome, xfce, etc.)
//This is determined from taking the ascii-lowercase form of a component
//the environment variable $XDG_CURRENT_DESKTOP, which is a colon-separated
//list of names that the current desktop is known as.
//get all existed mimeapps.list
std::vector<std::string> GetMimeappsLists();
//get the first writable mimeapps.list i.e. $XDG_CONFIG_HOME/mimeapps.list
std::string GetWritableMimeappsList();
//get the first writable current desktop specific mimeapps.list i.e. $XDG_CONFIG_HOME/$DESKTOP_SESSION-mimeapps.list
std::string GetWritableDesktopMimeappsList();

//get assigned desktop entry(s)
std::vector<FlyDesktopEntry> GetAssignedDesktopEntryForMime(const char *mime, AllDesktopEntries all = AllDesktopEntries{false});
//get icon(s) from assigned desktop entry
using AllIcons = Fly::tagged_bool<class AllIcons_tag>;
std::vector<std::string> GetAssignedIconForMime(const char *mime, AllIcons all = AllIcons{false});

//get list of all apps desktop entries paths
std::vector<std::string> GetAllAppDesktopEntryPaths();

//get list of all desktop entries search paths
std::vector<std::string> GetAllAppDesktopEntrySearchPaths();

// find folders, which contains .directory entry with non-empty Category field
std::vector<std::string> GetAllCategorizedFolderList();


//get list of all apps desktop entries
std::vector<FlyDesktopEntry> GetAllAppDesktopEntries();

//format exec string according to Desktop Entry spec
class ExecFormatString
{
public:
    ExecFormatString(const char *execFormat) : m_execFormat(execFormat) {}

    ExecFormatString &&filePath(const char *filePath) && { m_filePath = filePath; return std::move(*this); }
    ExecFormatString &&url(const char *url) && { m_url = url; return std::move(*this); }
    ExecFormatString &&icon(const char *icon) && { m_icon = icon; return std::move(*this); }
    ExecFormatString &&appName(const char *appName) && { m_appName = appName; return std::move(*this); }
    ExecFormatString &&desktopFilePath(const char *path) && { m_desktopFilePath = path; return std::move(*this); }
    ExecFormatString &&filePathList(std::vector<const char *> filePaths) && { m_filePathList = std::move(filePaths); return std::move(*this); }
    ExecFormatString &&urlList(std::vector<const char *> urls) && { m_urlList = std::move(urls); return std::move(*this); }
    ExecFormatString &&listSep(const char listSep) && { m_listSep = listSep; return std::move(*this); }

    operator std::string() && { return toString(); }

private:
    std::string toString();

    const char *m_execFormat;
    const char *m_filePath = nullptr;         // %f
    const char *m_url = nullptr;              // %u
    const char *m_icon = nullptr;             // %i
    const char *m_appName = nullptr;          // %c
    const char *m_desktopFilePath = nullptr;  // %k
    std::vector<const char *> m_filePathList; // %F
    std::vector<const char *> m_urlList;      // %U
    char m_listSep = ' ';
};
#endif

