#ifndef FLY_MISC_H
#define FLY_MISC_H

#include <fly/flytaggedbool.h>

#include <vector>
#include <string>

#include <limits.h>

using Display = struct _XDisplay;
using Window = unsigned long;

//mem total in kiloBytes and cpu in Mhz
unsigned int GetMemTotal();
unsigned int GetCPUMhz();
bool         lightPlatform();

//get system locale string, for example "ru_RU.KOI8-R"
const char *GetSystemLocale();
//get 2 symbols language prefix "ru", "en", etc. (for example to get qt translation file suffix)
std::vector<std::string> GetLangPref();

using IPv4Only = Fly::tagged_bool<class IPv4Only_tag>;
std::vector<std::string> GetIPAddrList(IPv4Only ipv4only = IPv4Only{true});

int  GetDisplayNumberByName(const char *display_name = nullptr, Display *dpy = nullptr);
bool isSessionRemote(const char *display_name = nullptr, Display *dpy = nullptr);
bool isSessionNested(const char *display_name = nullptr, Display *dpy = nullptr);

//find 'pattern' in 'ps -p uid' comand output
bool findInPS(const char *pattern);

//slow (who command used), to be replaced with fly-dmctl or direct utmp or ConsoleKit or systemd access
int numWho(bool *diffUsers);
int numUser(const std::string &user);

#define FlyTouchScreenDevice (1<<0)
#define FlyTouchPadDevice    (1<<1)
#define FlyPenDevice         (1<<2)
#define FlyEraserDevice      (1<<3)
#define FlyAllTouchTabletDevices (FlyTouchScreenDevice|FlyPenDevice|FlyEraserDevice|FlyTouchPadDevice)

struct FlyInputDevice
{
    std::string name;
    int id;
};

using AllDevices = Fly::tagged_bool<class AllDevices_tag>;
auto findTouchAndTabletDevices(Display *dpy, unsigned int type = FlyAllTouchTabletDevices, AllDevices all = AllDevices{true})
    -> std::vector<FlyInputDevice>;

bool isAlsaActive(); //check "snd" in 'lsmod' output or /proc/asound/cards content
//bool isKernel26();

std::string getUserName();
std::string getFullUserName();
bool isUserRoot();
bool isUserInGroup(const char *groupName = "astra-admin");
bool isUserInGroup(const char *groupName, const char *userName);
bool isUserInGroup(const char *groupName, uid_t userID);

bool isAstraSE(); //use it for 1.6/2.12 only, recommended to use getAstraMode() in all OS
int  getAstraMode(bool *ok=nullptr);

std::pair<int, int> wallpaperSizeRatio(int screenWidth, int screenHeight);

double GetXftDPI(Display *,int scr); //get current Xft dpi
using SetInXrdb = Fly::tagged_bool<class SetInXrdb_tag>;
void   SetXftDPI(Display *,int scr, double dpi, SetInXrdb xrdb); //set xft dpi, if xrdb=true then set it in RESOURSE_MANAGER too

int GetFontSizeForDPI(int dpi, int inc);

//set 1st randr screen as primary
bool SetRRPrimary(Display *, Window rootw, int randr_major=0, int randr_minor=0);
//get physical dpi of primary screen, but wrond EDID or nested X servers report 0mm x 0mm
bool GetRRPrimaryDPI(Display *,Window rootw, double *xdpi, double *ydpi);

//find real dpi via randr, x root window, xft
double FindDPI(Display *,int screen,Window root);

int getFreeXDisplayNumber(int startNum);

/*-------------------------------------------------------------*/

/* Result codes returned after parsing a command line. */
#define kMavNoErr					0
#define kMavErrUnbalancedQuotes		1
#define kMavErrTooManyQuotePairs	2
#define kMavErrTooManyReDirections	3
#define kMavErrNoReDirectedFileName	4
#define kMavErrNoPipeCommand		5
#define kMavErrNoReDirectedInput	6
#define kMavErrBothPipeAndReDirOut	7

/* The different sets of quotes supported. */
#define ISQUOTE(c) (((c) == '"') || ((c) == '\'') || ((c) == '`'))

/* Max number of sets of nested quotes supported. */
#define kMaxQuotePairs 15

/* Special token types. */
#define kRegularArg 0
#define kReDirectOutArg 1
#define kReDirectOutAppendArg 2

/* Limit on number of arguments to track. */
#define kMaxArgs 128

struct CmdLineInfo {
	int argCount;
	int isAppend;
	char outFileName[PATH_MAX];
	char pipeCmdLine[PATH_MAX];
	int savedStdout;
	int outFile;
	char argBuf[PATH_MAX*4/*ARG_MAX*/];  //ARG_MAX is 130000 - too big memory expenses!
	char *argVector[kMaxArgs + 4];
	int err;
	const char *errStr;
};

//#define NEWCMDLINEINFOPTR ((CmdLineInfoPtr) calloc((size_t)1, sizeof(CmdLineInfo)))

/* We put a few more things in the arg vector. */
#define CMDLINEFROMARGS(c,v) (v[(c) + 1])
#define CLIPFROMARGS(c,v) ((CmdLineInfoPtr) (v[(c) + 2]))

int MakeArgVector(char *str, CmdLineInfo *clp);
void ExpandDollarVariables(char *cmdLine, size_t sz);
/*-------------------------------------------------------------*/

#endif
