#ifndef FLY_SOUND_THEME
#define FLY_SOUND_THEME

#include <stdlib.h>

//#if defined(__linux__) || defined(_linux_) || defined(linux)
//#include <linux/limits.h> //for PATH_MAX, NAME_MAX
//#else
#include <limits.h>
//#endif

#include <fly/flyicontheme.h>

#include <vector>
#include <string>

#define SOUND_PROFILE_SIZE             30

//helper class for FlySoundTheme class
class FlySoundThemeSubdirPrivate;
class FlySoundThemeSubdir
{
public:
    FlySoundThemeSubdir(std::string path);
    FlySoundThemeSubdir(const FlySoundThemeSubdir &other) = delete;
    FlySoundThemeSubdir(FlySoundThemeSubdir &&other) noexcept;
    ~FlySoundThemeSubdir();

    FlySoundThemeSubdir &operator=(const FlySoundThemeSubdir &other) = delete;
    FlySoundThemeSubdir &operator=(FlySoundThemeSubdir &&other) noexcept;

    // set/get standard sound theme dir attributes
    const std::string &context() const;
    void setContext(std::string context);

    const std::string &profile() const;
    void setProfile(std::string profile);

    const std::string &path() const;
    void setPath(std::string path);

    // find full sound path by name (without extension!), locale, profile
    std::string findSound(const char *themePath, const char *soundName,
                          const char *locale, const char *soundProfile,
                          const std::vector<std::string>& spreadPaths) const;
private:
    FlySoundThemeSubdirPrivate * p;
};

class FlySoundThemePrivate;
class FlySoundTheme
{
public:
    FlySoundTheme(std::string name = std::string(), std::string parentName = std::string());
    FlySoundTheme(const FlySoundTheme &other) = delete;
    FlySoundTheme(FlySoundTheme &&other) noexcept;
    ~FlySoundTheme();

    FlySoundTheme &operator=(const FlySoundTheme &other) = delete;
    FlySoundTheme &operator=(FlySoundTheme &&other) noexcept;

    const std::string &path() const; // returns full path to index.theme file
    void setPath(std::string path);  // set     full path to index.theme file

    const std::string &themeName() const;     //get theme internal name
    void setThemeName(std::string name); //set theme internal name

    bool  load(); 		//reload index.theme file
    bool  save(); 		//save main attributes (see below) in index.theme file
    bool  isLoaded() const; //was it loaded at least once or not

    // set/get standard sound theme attributes

    const std::string &name() const; // get theme name
    void setName(std::string name);  // set theme name

    const std::string &localName() const; // get theme name in local language
    void setLocalName(std::string name);  // set theme name in local language

    const std::string &comment() const;   // get theme description
    void setComment(std::string comment); // set theme description

    const std::string &localComment() const;   // get theme description in local language
    void setLocalComment(std::string comment); // set theme description in local language

    bool hidden() const;  // true if theme is hidden (may be we shouldn't show it)
    void setHidden(bool);

    const std::string &exampleSound() const; // sound as theme example (usually sound name without extension)
    void setExampleSound(std::string sound); // sound as theme example (usually sound name without extension)

    const std::vector<std::string> &inheritedNames() const; //get inherited themes names (deep copy)
    void setInheritedNames(std::vector<std::string> names); //set inherited themes names (deep copy)

    const std::vector<FlySoundThemeSubdir> &subdirs() const; //get subdirs (deep copy)
    void setSubdirs(std::vector<FlySoundThemeSubdir> subdirs); //set subdirs (deep copy)

    //main API for apps: ---------------------------------------------

    // get sound theme path and description file name
    std::string findThemePath(const std::string &internalName);

    // find full sound path by sound short name (without extension!), by locale and by profile
    std::string getSoundFilePath(const char *soundNameIn, const char *localeIn, const char *profileIn);

//get dirs to search themes and sounds
static std::vector<std::string> GetThemesSearchDirs();

//fill possible themes list by pattern ("index.* or "cursors")
static std::vector<std::string> GetThemesIndexFiles();

//get name of current sound theme from ~/.fly/theme/current.themerc or some default    
static std::string getCurrentThemeName();
//create static object of current theme and return pointer to it
//forceCheck - check if sound theme was changed and create new one if so
using ForceCheck = Fly::tagged_bool<class ForceCheck_tag>;
using ForceLoad = Fly::tagged_bool<class ForceLoad_tag>;
static FlySoundTheme *createCurrentTheme(ForceCheck forceCheck = ForceCheck{false}, ForceLoad forceLoad = ForceLoad{false});

// profile and locale - may be NULL or ""
// fullPath- has to be non NULL field provided by caller
static std::string getSoundFilePathFromCurrentTheme(const char *soundName, const char *locale, const char *profile);

static std::vector<std::string> getThemesNames();

    // ---------------------------------------------------------------

private:
    FlySoundThemePrivate * p;
};

#endif

