#ifndef FLY_TRASH
#define FLY_TRASH

#include <string>

class FlyTrash
{
public:
    //register (not move file itself) file in trash
    //args:
    //  srcPath - full file/dir path to register in trash
    //  returns destination path to move file/dir to trash (i.e. ~/.local/share/Trash/files/name[_i])
    static std::string registerFile(const char *srcPath);

    //unregister file in trash
    //args:
    //  fileName - file/dir base name in trash dir (without path) or full path including trash path
    static bool unregisterFile(const char *fileName);

    //get file/dir path to restore from trash
    //args:
    //  fileNameOrPath - full file/dir path i.e. (GetFlyDir(USER_TRASH_DIR)/... or just file/dir name in trash dir without any '/'
    //  returns destination path to move file/dir from trash
    static std::string getFilePathToRestore(const char *fileNameOrPath);

    //get deletion date/time for file/dir in trash
    //args:
    //  fileNameOrPath - full file/dir path i.e. (GetFlyDir(USER_TRASH_DIR)/... or just file/dir name in trash dir without any '/'
    static time_t getDeletionDate(const char *fileNameOrPath);

    static bool isFileInTrash(const char * path);

    static bool cleanRegistry();//clean trash registry dir
    static bool cleanFiles();   //clean trash files dir
    static bool cleanAll();     //clean both above

    //get trash top dir path (i.e. ~/.local/share/Trash/)
    static std::string getTopDirPath();
    //get trash registry dir (i.e. ~/.local/share/Trash/info)
    static std::string getInfoDirPath();
    //get trash file dir (i.e. ~/.local/share/Trash/files)
    static std::string getFileDirPath();

    //check if trash empty (check files and info directories)
    static bool isEmpty();

    //real trash/untrash operations - not recommended due to 
    //  1) non-interactive (no progress indicators, no confirm dialogs)
    //  2) work inside the same mounted device only
    //  filePath - full path to file/dir to move to trash
    static bool moveToTrash(const char *filePath);
    //  fileNameOrPath - full file/dir path i.e. (GetFlyDir(USER_TRASH_DIR)/... or just file/dir name in trash dir without any '/'
    static bool restoreFromTrash(const char *fileNameOrPath);

    //NEW: get name of trashed file in trash dir,
    //  pathIn - full file or dir original path before trashed
    static std::string getFileNameInTrash(const char *pathIn);
    //NEW: get full path to trashed file in trash dir,
    //  pathIn - full file or dir original path before trashed
    static std::string getFilePathInTrash(const char *pathIn);

    static off_t size();
    static void registerDirSize(const char *dirName, off_t size = -1);
    static void unregisterDirSize(const char *dirName);
    static bool hasSizeCache();

private:
    static std::string setPath(const char *file, const char *path);
    static bool delPath(const char *fileName);
    static std::string getPath(const char *fileName);
    static std::string getDate(const char *fileName);
    static void registerDirSizeHelper(const char *dirName, off_t size);

};
#endif
